<?php
namespace Teknisa\CADFIS\Strategies;

use Teknisa\Libs\Util\Environment;
use Teknisa\CADFIS\Strategies\GeneralStrategy;
use Teknisa\CADFIS\Services\MarcaService as MarcaService;

class MarcaStrategy extends GeneralStrategy implements Interfaces\IStrategy
{
    protected $service;

    public function __construct(MarcaService $service, Environment $environment)
    {
		$this->service = $service;
		$this->environment = $environment;
	}
	
	public function validate(&$data)
    {
        foreach ($data as &$row) {
            parent::verifyIsEmpty($row, $row["ROWNUM"], ['CDMARCA', 'NMMARCA']);
            parent::verifyCdMarca($row, $row["ROWNUM"]);
            self::verifyMarcaExists($row, $row["ROWNUM"]);
            
            $row['ERROS'] = $this->errors;
            
            parent::clearErrors();
        }
    }
    
    private function verifyMarcaExists($row, $rowNum)
    {
        if ($this->service->verifyMarcaExists($row)) {
            parent::setError($rowNum, 'CDMARCA', $row['CDMARCA'], "ALREADY_REGISTERED");
        }
    }
    
    public function formatToView(&$data)
    {
        foreach ($data as &$row) {
           $row['IDERROR'] = !empty($row['ERROS']) ? 'S' : 'N';
        }
    }
    
    public function formatToSave(&$data)
    {
        foreach ($data as &$row) {
            $row['__is_new'] = true;
            $row['IDMARCAATIVA']    = 'A';
            $row['DTULTATUMARC']    = NULL;
            $row['NRORG']           = $this->environment->getNrorg();
            $row['DTINCLUSAO']      = NULL;
            $row['DTULTATU']        = NULL;
            $row['NRORGINCLUSAO']   = $this->environment->getNrorg();
            $row['CDOPERINCLUSAO']  = $this->environment->getCdOperador();
            $row['NRORGULTATU']     = $this->environment->getNrorg();
            $row['CDOPERULTATU']    = $this->environment->getCdOperador();
            $row['IDATIVO']         = 'S';
        } 
    }
    
    public function save($data)
    {
        self::validate($data);
        parent::verifyExistErros($data);
        $this->service->saveImport($data);
    }
}